/*
./config.js

This config.js JavaScript code is provided as part of the 
config.html document provided as part of the Doppler
RadLoop Desktop Gadget for Windows 7. 

All  rights not waved  by the author in the EULA are 
reserved by the author. This work may not be included 
in derivative works or modified in any  way.  This work  
can be freely  used and distributed in whole and 
unmodified provided that credit to the author is included. 
See eula.html for all details and limitations.

 jacksonapps.biz, 2010

Data and images displayed by this gadget are in the public domain
and are obtained from National Weather Service web sites at
http://radar.weather.gov and http://forecast.weather.gov.  This 
material is not subject to copyright protection (17 U.S.C. 403).
*/
			
// Set closing handler to local function to save settings
System.Gadget.onSettingsClosing=radarSettingsClosing;

// Define global variables used by the configuration  GUI 
var refreshRate;
var oLinkDiv;
var oInfoDiv;
var oRadarConfigDiv;
var oDispayConfigDiv;
var oLoopDiv;
var oEulaFrame;
var oEulaDiv;
var oCurrentDiv;
var oCurrentLink;
var startEulaState;
var eulaAccept="no";


function init() {
	// Determine if the EULA is already accepted and save starting EULA state
	startEulaState=System.Gadget.Settings.readString("eulaAccept");
	eulaAccept=startEulaState;
	if (eulaAccept!='yes') eulaAccept='no';
	
	// Read currently selected radar or default to MAF
	selectedRadar=System.Gadget.Settings.readString("radarSelected");
	if (selectedRadar == '') selectedRadar='MAF';
	for (i=0;i<userRadarSelect.options.length;i++) 
		if (userRadarSelect.options[i].value == selectedRadar) break;
	userRadarSelect.selectedIndex=i;
		
	// Read current color or use black as default
	selectedColor=System.Gadget.Settings.readString("colorSelected");
	if (selectedColor == '') selectedColor='black';
	for (i=0;i<userColorSelect.options.length;i++) 
		if (userColorSelect.options[i].value == selectedColor) break;									
	userColorSelect.selectedIndex=i;
					
	// Read current radar product or make Base Reflectivity the default
	selectedProduct=System.Gadget.Settings.readString("radarProduct");
	if (selectedProduct == '') selectedProduct='Base Reflectivity';
	for (i=0;i<userRadarProduct.options.length;i++) 
		if (userRadarProduct.options[i].value == selectedProduct) break;	
	userRadarProduct.selectedIndex=i;
	
	// Read width and default to 300 if not set
	radarWidth=System.Gadget.Settings.readString("radarWidth");
	if (!radarWidth) radarWidth=300;
	userRadarWidth.value=radarWidth;
	
	// Read refresh rate and default to 30 if not set
	refreshRate=System.Gadget.Settings.readString("refreshRate");
	if (!refreshRate) refreshRate=30;
	
	// Read default overlay states and set all but rivers on if not defined
	topoChecked=System.Gadget.Settings.readString("enableTopo");
	if (topoChecked) topoCheck.checked=((topoChecked=='on')?1:0);
	else topoCheck.checked=1;
	countyChecked=System.Gadget.Settings.readString("enableCounties");
	if (countyChecked) countyCheck.checked=((countyChecked=='on')?1:0);
	else countyCheck.checked=1;
	riverChecked=System.Gadget.Settings.readString("enableRivers");
	if (riverChecked) riverCheck.checked=((riverChecked=='on')?1:0);
	else riverCheck.checked=0;
	highwayChecked=System.Gadget.Settings.readString("enableHighways");
	if (highwayChecked) highwayCheck.checked=((highwayChecked=='on')?1:0);
	else highwayCheck.checked=1;
	cityChecked=System.Gadget.Settings.readString("enableCities");
	if (cityChecked) cityCheck.checked=((cityChecked=='on')?1:0);
	else cityCheck.checked=1;
	warningChecked=System.Gadget.Settings.readString("enableWarnings");
	if (warningChecked) warningCheck.checked=((warningChecked=='on')?1:0);
	else warningCheck.checked=1;
	
	// Read zip code for forecast or set to empty if not defined
	zipCode=System.Gadget.Settings.readString("zipCode");
	if (zipCode) userZipCode.value=zipCode
	else userZipCode.value='';
	
	// Read looping options
	startLooping=System.Gadget.Settings.readString("startLooping");
	if (startLooping) loopingCheck.checked=((startLooping=='yes')?1:0);
	else loopingCheck.checked=0;
	backfillLoop=System.Gadget.Settings.readString("backfillLoop");
	if (backfillLoop) backfillCheck.checked=((backfillLoop=='yes')?1:0);
	else backfillCheck.checked=0;
	maxLoop=System.Gadget.Settings.readString("maxLoop");
	if (!maxLoop) maxLoop=10;
	userMaxLoop.value=maxLoop;

	// Initialize main objects, particularly divs
	with (document) {
		oLinkDiv=getElementById('linkDiv');
		oInfoDiv=getElementById('infoDiv');
		oRadarConfigDiv=getElementById('radarConfigDiv');
		oDisplayConfigDiv=getElementById('displayConfigDiv');
		oLoopDiv=getElementById('loopConfigDiv');
		oEulaDiv=getElementById('eulaDiv');
		oEulaFrame=getElementById('eulaFrame');
	}						
		
	// Display EULA page if not accepted yet, otherwise default to radar
	if (eulaAccept=="yes") showPage('radarConfig');
	else showPage('eula');
}

function showPage(newSelection) {
	// Hide currently open page, if a page is open and deselect link
	if (oCurrentDiv) oCurrentDiv.style.display='none';
	if (oCurrentLink) oCurrentLink.className='';
	
	// If this is EULA page, show or hide extra text and buttons if accepted
	with (document) {
		// Make current division and link the one passed as newDivID
		oCurrentDiv=getElementById(newSelection+'Div');
		oCurrentLink=getElementById(newSelection+'Link');
		oCurrentLink.className='current';
	
		// If EULA is already accepted hide unnecessary buttons and text
		if (oCurrentDiv.id=='eulaDiv') {
			if (eulaAccept=='yes') {
				getElementById('mustAccept').innerHTML='';
				getElementById('acceptButton').style.display='none';
				getElementById('declineButton').style.display='none';
				oEulaDiv.style.top=17;
				oEulaFrame.style.height=250;
			}
		} 
	}
	
	// If EULA is accept show links at top
	if (eulaAccept=='yes') {
		oLinkDiv.style.display='block';
		oCurrentLink.style.display='block';
	}
	
	// Display current page
	oCurrentDiv.style.display='block';
	
	// Return false for links
	return false;
}

// If the EULA Accept button is clicked the acceptantance information is stored
// and the radarConfig page is activated
function acceptEULA() {
	eulaAccept='yes';
	System.Gadget.Settings.writeString("eulaAccept",eulaAccept);
	return showPage('info');
}

// When the EULA Decline button is clicked the gadget is completely closed
function declineEULA() {
	System.Gadget.close();
}

// Save radar settings on a settings closing event when commit button pressed
function radarSettingsClosing(event) {

	// Save settings only if Okay was clicked
	if ((startEulaState!=eulaAccept) || (event.closeAction == event.Action.commit)) {
		System.Gadget.Settings.writeString("radarSelected",userRadarSelect.options[userRadarSelect.selectedIndex].value);
		System.Gadget.Settings.writeString("colorSelected",userColorSelect.options[userColorSelect.selectedIndex].value);
		
		// Write big radar width, but restrict to 200-600 pixesl
		radarWidth=parseInt(userRadarWidth.value);
		if (radarWidth<200 || radarWidth>600) 	radarWidth=200;
		System.Gadget.Settings.writeString("radarWidth",radarWidth);
		
		// We are storing a refresh rate for now
		refreshRate=30;
		System.Gadget.Settings.writeString("refreshRate",refreshRate);
		
		// Store default radar product
		System.Gadget.Settings.writeString("radarProduct",userRadarProduct.options[userRadarProduct.selectedIndex].value);
		
		// Store default overlay states
		System.Gadget.Settings.writeString("enableTopo",topoCheck.checked?'on':'off');
		System.Gadget.Settings.writeString("enableCounties",countyCheck.checked?'on':'off');
		System.Gadget.Settings.writeString("enableRivers",riverCheck.checked?'on':'off');
		System.Gadget.Settings.writeString("enableHighways",highwayCheck.checked?'on':'off');
		System.Gadget.Settings.writeString("enableCities",cityCheck.checked?'on':'off');
		System.Gadget.Settings.writeString("enableWarnings",warningCheck.checked?'on':'off');
							
		// Store zip code for forecast if 5 characters
		zipCode=userZipCode.value?parseInt(userZipCode.value):'';
		if (zipCode.length<5) zipCode='';
		System.Gadget.Settings.writeString("zipCode",zipCode);
		
		// Store loop and backfill on start options
		System.Gadget.Settings.writeString("startLooping",(loopingCheck.checked?'yes':'no'));
		System.Gadget.Settings.writeString("backfillLoop",(backfillCheck.checked?'yes':'no'));
		
		// Store default loop length between 1 and 15		
		maxLoop=parseInt(userMaxLoop.value);
		if ((maxLoop>15) || (maxLoop<1)) maxLoop=10;
		System.Gadget.Settings.writeString("maxLoop",maxLoop);
	}
	
	// It is okay to close settings dialog now
	event.cancel = false;
}