/*
./main.js

This main.js JavaScript code is provided as part of the 
DopplerRadLoop.html document provided as part of 
the Doppler RadLoop Desktop Gadget for Windows 7. 

All  rights not waved  by the author in the EULA are 
reserved by the author. This work may not be included 
in derivative works or modified in any  way.  This work  
can be freely  used and distributed in whole and 
unmodified provided that credit to the author is included. 
See eula.html for all details and limitations.

 jacksonapps.biz, 2010

Data and images displayed by this gadget are in the public domain
and are obtained from National Weather Service web sites at
http://radar.weather.gov and http://forecast.weather.gov.  This 
material is not subject to copyright protection (17 U.S.C. 403).
*/
// The following define where to find each type of image
var radarURL='http://radar.weather.gov/ridge/radar.php';
var forecastURL='http://forecast.weather.gov/zipcity.php';
var ridgeBase='http://radar.weather.gov/ridge';
var radarBase=ridgeBase+'/RadarImg';
var loopBase=ridgeBase+'/RadarImg';
var topoBase=ridgeBase+'/Overlays/Topo';
var countyBase=ridgeBase+'/Overlays/County';
var riverBase=ridgeBase+'/Overlays/Rivers';
var highwayBase=ridgeBase+'/Overlays/Highways';
var cityBase=ridgeBase+'/Overlays/Cities';
var warningBase=ridgeBase+'/Warnings';

// The following URLs are determined by when radar selected
var radarSrc;
var loopSrc;
var gfwSrc;
var warningSrc;
var warningLoopSrc;

// EULA acceptance flag
var eulaAccept='no';
var lastEula='no';

// User settings
var productSelected;
var radarSelected;
var radarWidth;
var radarHeight;
var imgRadarWidth=600.0;
var imgRadarHeight=550.0;
var imgRadarRatio=imgRadarHeight/imgRadarWidth;
var radarDivImgRatio;
var radarRange;
var radarProdId;
var radarProdPub;
var colorSelected;
var linksTop;

// The following are objects referenced in the script
var oBody;
var oRadarBackground;
var oNoDataDiv;
var oOldDataDiv;
var oRadarDiv;
var oLinkContainer;
var oLoopDiv;
var oInfo;
var oLinkShadow;
var oForecastLink;
var oRadarLink;
var oTopoImg;
var oRadarImg;
var oCountyImg;
var oRiverImg;
var oHighwayImg;
var oCityImg;
var oWarningImg;
var oPlayToggle;
var oNoDataReason;
var oUpgradeButton;
var oHelpButton;
var oFlyoutDoc;

// Current settings
var cHeight;
var cWidth;
var cInfoUp;
var cInfoDown;
var cWinScale;

var helpShowing=false;

// Dock and undock are for expanding view temporarily
var dragging;
var initMouseEvent;
var downX;
var downY;
var selectBoxToggle=true;
var oZoomArray=new Array();
var oCurrentBox=new Array();
var oGfwInfo=new Array();
var maxZoom=12.0;
var totalZoom=1.0;

// Create objects for radar product list, warning list, backgrounds and upgrade requests
var oRadarRequest = new XMLHttpRequest();
var oGfwRequest = new XMLHttpRequest();
var oVersionRequest = new XMLHttpRequest();
var oRadarFilePatt=/(([A-Z]{3})_((\d{4})(\d{2})(\d{2})_(\d{2})(\d{2})))_([A-Z0-9]{3})\.gif/i;
oRadarFilePatt.compile(oRadarFilePatt);

// Create radar loop and update-related objects and related variables
var oLoopRadar=new Array();
var maxLoop;
var loopIndex=0;
var loopSpeed=400;
var loopTimer;
var warningTimer;
var mouseOutTimer;
var lastRadarReq=0;
var firstUpdateOccurred;
var updateAttempt;
var updateInterval;
var updateTimer;
var lastRadarLoaded='';
var lastConfirmedConnect=0;
var forceOverlayLoad;
var radarUpdating=false;
var isLooping;
var lastLoopState;
var backfillLoop;
var lastClickTime=0;
var thisVersion=System.Gadget.version;
var versionSrc='http://www.jacksonapps.biz/DopplerRadLoop/download/version.txt';
var gadgetHome='http://www.jacksonapps.biz/DopplerRadLoop/';
var lastVersionCheck=0;
var newAvailable='';
			
// Initialize the gadget.
function init() {
	with (document) {
		oBody = body;
		oRadarBackground = getElementById("radarBackground");
		oRadarContainer = getElementById("radarContainer");
		oRadarDiv = getElementById("radarDiv");
		oTopoImg = getElementById("topoImg");
		oRadarImg = getElementById("radarImg");
		oCountyImg = getElementById("countyImg");
		oRiverImg = getElementById("riverImg");
		oHighwayImg = getElementById("highwayImg");
		oCityImg = getElementById("cityImg");
		oWarningImg = getElementById("warningImg");
		oSelectBox = getElementById("selectBox");
		oNWSLinks = getElementById("nwsLinks");
		oRadarLink = getElementById("radarLink");
		oForecastLink = getElementById("forecastLink");
		oLinkContainer = getElementById("linkContainer");
		oLinkShadow = getElementById("linkShadow");
		oLoopDiv = getElementById("loopDiv");
		oInfo=getElementById("info");
		oNoDataDiv=getElementById("noDataDiv");
		oOldDataDiv=getElementById("oldDataDiv");
		oNoDataReason=getElementById("noDataReason");
		oPlayToggle=getElementById("playToggleButton");
		oUpgradeButton=getElementById("upgradeButton");
		oHelpButton=getElementById("helpButton");
	}
	
	// Enable settings dialog
	System.Gadget.settingsUI = "./config.html";
	System.Gadget.onSettingsClosed = radarSettingsClosed;
	System.Gadget.onShowSettings = radarSettingsOpened;
	
	// Setup flyout (used for help)
	System.Gadget.Flyout.file = "./helpflyout.html";
	System.Gadget.Flyout.onShow=flyoutShowing;
	System.Gadget.Flyout.onHide=flyoutHiding;
	
	// Read last settings
	readRadarSettings(true);
	
	// Set radar div mouse event
	oRadarDiv.onmousedown=handleMouseDown;
	
}

function readRadarSettings(forceUpdate) {
	// Check EULA and force complete update if just accepted
	eulaAccept=System.Gadget.Settings.readString("eulaAccept");
	if (eulaAccept=='yes')
		oNoDataReason.innerHTML='<br />Internet Available?';
	forceUpdate=(forceUpdate || (lastEula !=eulaAccept));
			
	// If the settings for the background changed or update forced
	lastColor=colorSelected;
	colorSelected=System.Gadget.Settings.readString("colorSelected");
	if (lastColor!=colorSelected || forceUpdate) {
		if (colorSelected=='') {
			colorSelected='black';
			System.Gadget.Settings.writeString("colorSelected",colorSelected);
		}
		oRadarBackground.src='images/'+colorSelected+'.gif';
	}
	
	// Read the update interval
	updateInterval=System.Gadget.Settings.readString("refreshRate");
	if (!updateInterval) updateInterval=120;
	
	// Do we backfill on first load?
	lastBackfillLoop=backfillLoop;
	backfillLoop=(System.Gadget.Settings.readString("backfillLoop")=='yes')?1:0;
	forceUpdate=forceUpdate || (lastBackfillLoop != backfillLoop);
						
	// If the settings for the radar changed or update forced load new radar
	lastRadar=radarSelected;
	radarSelected=System.Gadget.Settings.readString("radarSelected");
	forceUpdate=forceUpdate ||  (lastRadar != radarSelected);
	lastRange=radarRange;
	lastProduct=productSelected;
	productSelected=System.Gadget.Settings.readString("radarProduct");
	forceUpdate=forceUpdate || (lastProduct != productSelected);
	if (forceUpdate) {
		// Hide radar for transition
		oRadarImg.style.visibility='hidden';
	
		// Set forceUpdate for other condition tests that are affected
		forceUpdate=true;
		
		oNoDataDiv.style.visibility='visible';
		
		if (radarSelected=='') {
			radarSelected='MAF';
			System.Gadget.Settings.writeString("radarSelected",radarSelected);
		}
		
		radarRange='Short';
		switch (productSelected) {
			case 'Long Range Reflectivity':
				radarProdId='N0Z';
				radarProdPub='LR';
				radarRange='Long';
				break;
			case 'Storm Relative Motion':
				radarProdId='N0S';
				radarProdPub='SRM';
				break;
			case 'Base Velocity':
				radarProdId='N0V';
				radarProdPub='V';
				break;
			case 'One-Hour Precipitation':
				radarProdPub='OHP';
				radarProdId='N1P';
				break;
			case 'Composite Reflectivity':
				radarProdPub='CR';
				radarProdId='NCR';
				break;
			case 'Storm Total Precipitation':
				radarProdPub='STP';
				radarProdId='NTP';
				break;							
			case 'Base Reflectivity':
			default:
				radarProdPub='R';
				radarProdId='N0R';
		}
		
		// Assign an image to the radar and make visible
		lastRadarLoaded='';
		firstUpdateOccurred=false;
		radarSrc=radarBase+'/'+radarProdId+'/'+radarSelected+'_'+radarProdId+'_0.gif';
		loopSrc=loopBase+'/'+radarProdId+'/'+radarSelected;
		warningSrc=warningBase+'/'+radarRange+'/'+radarSelected+'_Warnings_0.gif';
		warningLoopSrc=warningBase+'/'+radarRange+'/'+radarSelected;
		gfwSrc=radarBase+'/'+radarProdId+'/'+radarSelected+'_'+radarProdId+'_0.gfw';
		oGfwInfo=new Array();
		
		// Assign the topo and overlay images and make visible
		oTopoImg.src='.mages/blank.gif';
		oTopoImg.src=topoBase+'/'+radarRange+'/'+radarSelected+'_Topo_'+radarRange+'.jpg';
		oCountyImg.src='images/blank.gif';
		oRiverImg.src='images/blank.gif';
		oCityImg.src='images/blank.gif';
		oHighwayImg.src='images/blank.gif';
		oRiverImg.src=riverBase+'/'+radarRange+'/'+radarSelected+'_Rivers_'+radarRange+'.gif';
		oCountyImg.src=countyBase+'/'+radarRange+'/'+radarSelected+'_County_'+radarRange+'.gif';
		oHighwayImg.src=highwayBase+'/'+radarRange+'/'+radarSelected+'_Highways_'+radarRange+'.gif';
		oCityImg.src=cityBase+'/'+radarRange+'/'+radarSelected+'_City_'+radarRange+'.gif';
		
		// Force reload of images
		forceOverlayLoad=true;
	}
	
	// Scale the divs
	lastWidth=radarWidth;
	radarWidth=parseInt(System.Gadget.Settings.readString("radarWidth"));
	if (!radarWidth)
		radarWidth=250;
	radarDivImgRatio=(1.0*radarWidth)/imgRadarWidth;
	radarHeight=parseInt(1.0*radarWidth*imgRadarHeight/imgRadarWidth);
	
	// Size the gagdet and scale if not already grabbed
	if (!cWinScale) gadgetSize();

	if ( (oCurrentBox.length==0) || (radarRange!=lastRange) || (radarSelected!=lastRadar) || (radarWidth!=lastWidth )) {
		totalZoom=1.0;
		oCurrentBox=Array(0,0,radarWidth,radarHeight,totalZoom);
		oZoomArray=Array();
	}
	
	// On initial start, read last zoom
	if (forceUpdate) {
		totalZoomInfo=parseFloat(System.Gadget.Settings.readString("totalZoom"));
		if (totalZoomInfo) {
			totalZoom=totalZoomInfo;
			cBox0=parseInt(System.Gadget.Settings.readString("cBox0"));
			cBox1=parseInt(System.Gadget.Settings.readString("cBox1"));
			cBox2=parseInt(System.Gadget.Settings.readString("cBox2"));
			cBox3=parseInt(System.Gadget.Settings.readString("cBox3"));
			oZoomArray.push(oCurrentBox);
			oCurrentBox=Array(cBox0,cBox1,cBox2,cBox3,totalZoom);
		}
	}
			
	radarOverlays=resetOverlaySettings(true);

	// Add radar and forecast links
	radarArgs='rid='+radarSelected+'&product='+radarProdId+'&overlay='+radarOverlays+'&loop=yes';
	oRadarLink.innerHTML='<a href="'+radarURL+'?'+radarArgs+'">Radar</a>';
	zipCode=System.Gadget.Settings.readString("zipCode");
	if (zipCode) {
		forecastArgs='inputstring='+zipCode;
		oForecastLink.innerHTML='<a href="'+forecastURL+'?'+forecastArgs+'">Fcst</a>';
	} else {
		oForecastLink.innerHTML='';
	}
	
	if (forceUpdate) oLoopRadar=new Array();
	
	// Set docking actions
	System.Gadget.onDock=dockAction;
	System.Gadget.onUndock=dockAction;
	
	if (eulaAccept!='yes') return;
	
	// Get maximum loop length
	maxLoop=System.Gadget.Settings.readString("maxLoop");
	if (!maxLoop) maxLoop=10;
	
	// Request first radar images
	requestRadarUpdate();
	updateTimer=setTimeout(requestRadarUpdate,updateInterval*1000);
	
	// Are we looping?
	loopingStat=System.Gadget.Settings.readString("startLooping");
	if ((!loopingStat || (loopingStat=="yes")) && (maxLoop>1))  loopOn();
	else loopOff();
	
	// Zoom to correct level and turn on radar image
	zoomTo();
	oRadarImg.style.visibility='visible';
	
	// Set visibility change options
	System.Gadget.visibilityChanged=checkVisibility;
}

/* When the gadget visibility state changes, modify loop behavior */
function checkVisibility() {
	// If gadget became visible, continue last loop state and request radar update
	if (System.Gadget.visible) {
		if (System.Gadget.Settings.readString("startLooping")=='yes') loopOn();
		else tail();
		requestRadarUpdate();
	} else {
		loopOff();
		if (updateTimer) clearTimeout(updateTimer);
	}
}

function  gadgetSize()  {
	// Determine new dimensions and window scaling based on docking state
	if (System.Gadget.docked) {
		cWidth=130;
		cHeight=119;
		cWinScale=(1.0*radarWidth)/cWidth;
	} else {
		cWidth=radarWidth;
		cHeight=radarHeight;
		cWinScale=1.0;
	} 
}
	
function dockAction() {
	// Begin transition and set background width to zero
	System.Gadget.beginTransition();
	oRadarBackground.style.width=0;
	
	gadgetSize();
	
	// Rescale body
	oBody.style.width=cWidth;
	oBody.style.height=cHeight;
	
	// Determine base font size for links box
	if (cWidth>200) {
		oBody.style.fontSize='15pt';
		oLoopDiv.className='large';
		oHelpButton.className='buttonLarge'
		oUpgradeButton.className='buttonLarge'
	} else  {
		oBody.style.fontSize='11pt';
		oLoopDiv.className='small';
		oHelpButton.className='buttonSmall'
		oUpgradeButton.className='buttonSmall'
	}
	
	// Rescale background and radar container
	oRadarBackground.style.width=cWidth;
	oRadarBackground.style.height=cHeight;
	oRadarContainer.style.width=cWidth;
	oRadarContainer.style.height=cHeight;
	
	// Set width of links box, the loop button div first due to possible effect on height
	oLinkShadow.style.width=cWidth;
	oLinkContainer.style.width=cWidth-2;
	oNWSLinks.style.width=cWidth-2;
	oLoopDiv.style.width=cWidth-34;
	
	// Set heights based on links div height
	linksHeight=oNWSLinks.offsetHeight+3;
	oLinkShadow.style.height=linksHeight;
	oLinkContainer.style.height=linksHeight;
	
	// Finally set top of divs
	cLinksTop=cHeight-linksHeight;
	oLinkShadow.style.top=cLinksTop;
	oLinkContainer.style.top=cLinksTop;
		
	// Set height of main info (time) line
	cInfoUp=cHeight-linksHeight;
	cInfoDown=cHeight-oInfo.offsetHeight;
		
	// Initially hide links
	hideNWSLinks();
	
	// Refocus on correct area
	zoomTo();
	
	 // End transition using morph transition type
	System.Gadget.endTransition(System.Gadget.TransitionType.morph,3);
}

function resetOverlaySettings(allFlag) {
	radarOverlays='';
	if (allFlag) {
		topoChecked=System.Gadget.Settings.readString("enableTopo");
		topoChecked=!topoChecked || topoChecked=='on';
		oTopoImg.style.visibility=topoChecked?'visible':'hidden';
		document.getElementById('checkTopo').tag=topoChecked;
		document.getElementById('checkTopo').className=topoChecked?'overlayOn':'overlayOff';
		radarOverlays+=topoChecked?'1':'0';
	}
				
	radarOverlays+='1';
	
	checkState=false;
	
	countyChecked=System.Gadget.Settings.readString("enableCounties");	
	countyChecked=!countyChecked || countyChecked=='on';
	oCountyImg.style.visibility=countyChecked?'visible':'hidden';
	radarOverlays+=countyChecked?'1':'0';
	checkState=checkState||countyChecked;
	
	riverChecked=System.Gadget.Settings.readString("enableRivers");	
	riverChecked=(riverChecked=='on');
	oRiverImg.style.visibility=riverChecked?'visible':'hidden';
	radarOverlays+=riverChecked?'1':'0';
	checkState=checkState||riverChecked;
	
	highwayChecked=System.Gadget.Settings.readString("enableHighways");		
	highwayChecked=!highwayChecked || highwayChecked=='on';
	oHighwayImg.style.visibility=highwayChecked?'visible':'hidden';
	checkState=checkState||highwayChecked;
	radarOverlays+=highwayChecked?'1':'0';
	
	cityChecked=System.Gadget.Settings.readString("enableCities");		
	cityChecked=!cityChecked || cityChecked=='on';
	oCityImg.style.visibility=cityChecked?'visible':'hidden';
	checkState=checkState||cityChecked;
	radarOverlays+=cityChecked?'1':'0';
	
	document.getElementById('checkOverlays').tag=checkState;
	document.getElementById('checkOverlays').className=checkState?'overlayOn':'overlayOff';
	
	if (allFlag) {
		warningChecked=System.Gadget.Settings.readString("enableWarnings");	
		warningChecked=!warningChecked || warningChecked=='on';
		oWarningImg.style.visibility=warningChecked?'visible':'hidden';
		document.getElementById('checkWarnings').tag=warningChecked;
		document.getElementById('checkWarnings').className=warningChecked?'overlayOn':'overlayOff';
		radarOverlays+=warningChecked?'1':'0';
	}
	
	// Include legend
	radarOverlays+='1';
	return radarOverlays;
}

function requestGfwInfo() {
	oGfwInfo=new Array();
	oGfwRequest.open("GET",gfwSrc,true);
	oGfwRequest.onreadystatechange = handleGfwResponse;
	oGfwRequest.send();
}

function handleGfwResponse() {
	if (oGfwRequest.readyState == 4 ) {
		if (oGfwRequest.status == 200 ) {
			oGfwInfo=oGfwRequest.responseText.split('\n');
		}
	}
}

function versionCheck() {
	epoch=new Date();
	oVersionRequest.open("GET",versionSrc+'?T='+epoch.getTime(),true);
	oVersionRequest.onreadystatechange = handleVersionResponse;
	oVersionRequest.send();
}

function handleVersionResponse() {
	if (oVersionRequest.readyState == 4 ) {
		if (oVersionRequest.status==200) {
			// Base update on build number only, not full version
			currentVersion=oVersionRequest.responseText.replace(/\s+.*/,'');
			currentBuild=parseInt(currentVersion.replace(/.*\./,''));
			thisBuild=parseInt(thisVersion.replace(/.*\./,''));
			if (currentBuild > thisBuild) {
				newAvailable=currentVersion;
				oUpgradeButton.style.visibility='visible';
			}
		}
	}
}

function launchSite() {
	window.open("http://www.jacksonapps.biz/DopplerRadLoop/?version="+currentVersion);
}

function requestRadarUpdate() {
	epoch=new Date();
	lastRadarReq=epoch.getTime();
	dirURL=loopSrc+'/?C=N&F=0&T='+epoch.getTime();
	oRadarRequest.open("GET",dirURL+'?T='+epoch.getTime(),true);
	oRadarRequest.onreadystatechange = handleRadarResponse;
	oRadarRequest.send();
}

function handleRadarResponse() {
	if (oRadarRequest.readyState == 4 ) {
		if (oRadarRequest.status==200) {
			// We will use the current time several times here
			nowTime=new Date();
			
			// 200 means that we must have internet
			lastConfirmedConnect=nowTime.getTime();
		
			origLoopSize=oLoopRadar.length;
			radarUpdating=true;
			radarListing=new Array();
			radarListing=oRadarRequest.responseText.split('\n');
			radarProducts=new Array();
			nFound=0;
			
			while ((radarListing.length>0) && (nFound<maxLoop)) {
				
				line=radarListing.pop();
				matchingFile=oRadarFilePatt.exec(line);
				
				if (matchingFile && (matchingFile[0]>lastRadarLoaded)) {
					var timeStamp=new Date();
					with (timeStamp) {
						timeStamp.setUTCFullYear(matchingFile[4]);
						timeStamp.setUTCMonth(matchingFile[5]-1);
						timeStamp.setUTCDate(matchingFile[6]);
						timeStamp.setUTCHours(matchingFile[7]);
						timeStamp.setUTCMinutes(matchingFile[8]);
					}
					matchInfo=new Array(matchingFile[0],matchingFile[1],'K'+matchingFile[2],timeStamp);
					radarProducts.unshift(matchInfo);
					nFound++;
				}
			}
		
			if (nFound>0) {
				while (radarProducts.length>0) {
					newRadar=radarProducts.shift();
					
					// Skip new images on initial load if backfill option is not checked
					if (!firstUpdateOccurred && !backfillLoop && (radarProducts.length>0)) continue;
										
					newRadar[4]=loopSrc+'/'+newRadar[0];
					oImg=new Image(radarWidth,radarHeight);
					oImg.src=newRadar[4];

					warnURL=warningLoopSrc+'/'+newRadar[1]+'_Warnings.gif';
					newRadar[5]=warnURL;
					
					hour=newRadar[3].getHours();
					ampm='a';
					if (hour>=12) {
						hour-=12;
						ampm='p';
					}
					if (hour==0) hour=12;
					minute='00'+newRadar[3].getMinutes();
					minute=minute.substring(minute.length-2);
					newRadar[6]=hour+':'+minute+ampm+' '+radarProdPub;

					oLoopRadar.push(newRadar);
					lastRadarLoaded=newRadar[0];
					if (oLoopRadar.length>maxLoop) {
						oLoopRadar.shift();
						firstUpdateOccurred=true;
					}
					
					// If not looping, need to run loopStep in a second
					// to position loopIndex at end after timeout of 5 minutes
					if (!isLooping) {
						loopIndex=loopIndex-nFound;
						loopTimer=setTimeout(loopStep,1000);
					}
						
				}
			}
			newImageFound=(nFound>0);
			nextUpdate=nextUpdateInterval(true,newImageFound)
			radarUpdating=false;
			
			// If no GFW info yet, request it
			if (oGfwInfo.length==0) requestGfwInfo();
		
			// Hide missing data indicator if data arrived and make sure each map loaded
			if (forceOverlayLoad) {
				oTopoImg.src=oTopoImg.src+'?reload=1';
				oCountyImg.src=oCountyImg.src+'?reload=1';
				oRiverImg.src=oRiverImg.src+'?reload=1';
				oHighwayImg.src=oHighwayImg.src+'?reload=1';
				oCityImg.src=oCityImg.src+'?reload=1';
				oNoDataDiv.style.visibility='hidden';
				forceOverlayLoad=false;
			}
			
			// Update last warning
			warningUpdate();
			
			// Check for new version if not already found and two days have elapsed since last check
			if (!newAvailable && ((nowTime.getTime()-lastVersionCheck)>172800000)) {
				lastVersionCheck=nowTime.getTime();
				versionCheck();
			}
				
		} else {
			nextUpdate=nextUpdateInterval(false,false);
		}
		oRadarRequest.onreadystatechange=null;
		updateTimer=setTimeout(requestRadarUpdate,nextUpdate);
	}
}

function nextUpdateInterval(lastSuccess,newImageLoaded) {
	// Good time to check if latest data is old
	nowTime=new Date();
	if ((oLoopRadar.length==0) || (nowTime.getTime()-oLoopRadar[oLoopRadar.length-1][3].getTime()>900000)) {
		if (nowTime.getTime() - lastConfirmedConnect>600000)
			oOldDataDiv.innerHTML='Radar Data Old<br />No Internet';
		else 
			oOldDataDiv.innerHTML='Radar Data Old';
		oOldDataDiv.style.visibility='visible';
	} else
		oOldDataDiv.style.visibility='hidden';
	
	// Use user update interval when less than two images loaded				
	if (oLoopRadar.length<2) {
		nextUpdate=updateInterval;
	} else {
		// If we got a radar listing use algorithm to determine next update
		if (lastSuccess) {
			// If new image was loaded, assume interval between last two will repeat
			if (newImageLoaded && firstUpdateOccurred) {
				radDiff=Math.round((oLoopRadar[oLoopRadar.length-1][3].getTime()-oLoopRadar[oLoopRadar.length-2][3].getTime())/1000);
				nextUpdate=radDiff-2*updateInterval;
				if (nextUpdate<updateInterval) nextUpdate=updateInterval;
				updateAttempt=0;
			
			// If no new image after last check, start using user interval
			} else {
				updateAttempt++;
				
				// If no images after 5 tries, retry update in 5 minutes
				if (! firstUpdateOccurred || (updateAttempt<6))
					nextUpdate=updateInterval;
				else
					nextUpdate=300;
			}
			
		// Wait five minutes if last request for update failed
		} else {
			nextUpdate=300;
		}
	}
	
	// Return seconds multiplied by 1000 for milliseconds
	return (nextUpdate*1000);
}

function warningUpdate() {
	nowTime=new Date;
	if (warningTimer) clearTimeout(warningTimer);
	
	// We only load a warning image if we have at least one radar image
	lastIndex=oLoopRadar.length-1;
	if (!radarUpdating && (lastIndex>=0)) {
		warnURL=warningSrc+'?T='+nowTime.getTime();
		oLoopRadar[lastIndex][5]=warnURL;
		
		// If we are at the last image in the loop, load the current warning image
		if (loopIndex==lastIndex)  oWarningImg.src=warnURL;
	}
	
	// Check for a new image in 60 seconds
	warningTimer=setTimeout(warningUpdate,60000);
	
	// If the last radar request was more than 15 minutes ago, we need
	// request now (probably just woke up)
	if (lastRadarReq && ((nowTime.getTime()-lastRadarReq)>900)) {
		if (updateTimer) clearTimeout(updateTimer);
		requestRadarUpdate();
	}
}

/* When the radar settings are opened, clear all timeouts  and disable loop */
function radarSettingsOpened(event) {
	if (updateTimer) 
		clearTimeout(updateTimer);
	if (warningTimer)
		clearTimeout(warningTimer);
	lastLoopState=isLooping;
	loopOff();
	
	// Store current EULA accept state
	lastEula=eulaAccept;
}

/* When the settings dialog closes, we need to check EULA state */
function radarSettingsClosed(event) {
	// Check EULA regardless of whether commited and force
	eulaAccept=System.Gadget.Settings.readString("eulaAccept");

	// Don't do anything new unless the last action was commited
	if (event.closeAction == event.Action.commit) 
		readRadarSettings(false);
	else {
		if  (eulaAccept=='yes') {
			if (lastEula!=eulaAccept) readRadarSettings(false);
			else {
				requestRadarUpdate();
				if (lastLoopState) loopOn();
			}
		}
	}
}

/*	When mouse is pressed with Ctrl, Shift, or Alt we trap the event
	for zoom, shift or forecast on mouse move or mouse up. Position
	is relative to radarDiv */
function handleMouseDown() {	
	// Flag initial event contol key type that began mouse press
	// or return if no control key
	if (event.ctrlKey)	initMouseEvent='ctrl';
	else if (event.shiftKey) initMouseEvent='shift';
	else if (event.altKey) initMouseEvent='alt';
	else {
		event.cancelBubble=false;
		newEvent=document.createEventObject(event);
		parent.document.fireEvent('onmousedown',newEvent);
		event.returnValue=true;
		return true;
	}
		
	// Get the initial mouse-down position
	downX=event.offsetX;
	downY=event.offsetY;
		
	// Enable dragging state for ctrl or shift and set handler
	dragging=(initMouseEvent!='alt')?1:0;
	if (dragging) {
		// Hide bottom info during drag events
		oLinkContainer.style.visibility='hidden';
		oLinkShadow.style.visibility='hidden';
		
		// Enable mouse move events for dragging
		oRadarDiv.onmousemove=handleMouseMove;
	}
	
	// We need to trap mouse up and mouse leave regardless of control key
	oRadarDiv.onmouseup=handleMouseUp;
	oRadarDiv.onmouseleave=handleMouseOut;
}

/*	When the mouse has moved after trapped mouseDown event
	update selection box within radarDiv based on the new position.
	this event will only be trapped for Ctrl or Shift.*/
function handleMouseMove(fromObj) {
	// If the sending object was the selection box, ignore event
	srcElementId=event.srcElement.id;
	if (srcElementId=='selectBox') return false;
	
	// Disable mouse move events while performing drag action to prevent overlapping events
	oRadarDiv.onmousemove=null;
		
	// Get new mouse position
	x=event.offsetX;
	y=event.offsetY;
	
	// Handle selection box for zoom if Ctrl key down
	if (event.ctrlKey && initMouseEvent=='ctrl') {
		// Only begin selection box if mouse has move a bit
		mouseDelta=Math.sqrt(Math.pow(x-downX,2)+Math.pow(y-downY,2));
		if (mouseDelta>30.0) {
		
			// Transition to selection box dragging state
			dragging=2;
			oSelectBox.style.visibility='visible';
	
			// Determine top and bottom of selected box
			if (x>downX) {
				topX=downX;
				bottomX=x;
			} else {
				topX=x;
				bottomX=downX;
			}
			if (y>downY) {
				topY=downY;
				bottomY=y;
			} else {
				topY=y;
				bottomY=downY;
			}
	
			// Redraw selection box based on new mouse position
			oSelectBox.style.top=topY;
			oSelectBox.style.left=topX;
			oSelectBox.style.height=Math.abs(bottomY-topY)+1;
			oSelectBox.style.width=Math.abs(bottomX-topX)+1;
		}
	} else if (event.shiftKey && (initMouseEvent=='shift')) {
		// Calculate amount box should be shifted based on 
		// mouse movement
		diffX=x-downX;
		diffY=y-downY;
		newLeft=oCurrentBox[0]+diffX;
		newTop=oCurrentBox[1]+diffY;
		newRight=newLeft+oCurrentBox[2]-1;
		newBottom=newTop+oCurrentBox[3]-1;
		
		// If we can't drag any further, just record mouse position 
		if ((newLeft>0) || (newRight<radarWidth) || (newTop>0) || (newBottom<radarHeight)) {
			//endDragging();
			//initMouseEvent='';
			downX=x;
			downY=y;
			
		// If we are within bounds go to new calculated location
		} else {
			oCurrentBox[0]=newLeft;
			oCurrentBox[1]=newTop;
			zoomTo();
		}
	}  else {
		// When control key status has changed, end dragging
		endDragging();
		initMouseEvent='';
		return;
	}
	oRadarDiv.onmousemove=handleMouseMove;
}

/*	When the mouse is released after being clicked with either
	Ctrl or Alt key down, we need to perform zoom action or open a
	forecast window for the selected point .  For Shift key, we
	just end dragging */
function handleMouseUp() {
	// Capture end drag state and end dragging
	endDragState=dragging;
	endDragging();
	
	// Get final mouse position
	x=event.offsetX;
	y=event.offsetY;
	if (event.ctrlKey && (initMouseEvent=='ctrl')) {
		// Determine whether to zoom or unzoom based on mouse state
		if (endDragState==2) radarZoom();
		else radarUnZoom();
	} else if (event.altKey && (initMouseEvent=='alt')) {
		// Assume initial mouse down position is intended forecast point
		x=downX;
		y=downY;
		
		// The actual pixel offset is determined by scaling by zoom factors and radarDiv size
		radarOffsetX= cWinScale*(1.0*x/totalZoom)/radarDivImgRatio;
		radarOffsetY= cWinScale*(1.0*y/totalZoom)/radarDivImgRatio;
		lon=parseInt((parseFloat(oGfwInfo[4])+radarOffsetX*parseFloat(oGfwInfo[0]))*100)/100;
		lat=parseInt((parseFloat(oGfwInfo[5])+radarOffsetY*parseFloat(oGfwInfo[3]))*100)/100;
		forecastURL='http://forecast.weather.gov/MapClick.php?lat='+lat+'&lon='+lon;
		window.open(forecastURL);
	}
	
	// Make sure that old mouse event doesn't affect next event processing
	initMouseEvent='';
}

/* Deal with mouse leaving radar, give 2 seconds to get back in box */
function handleMouseOut() {
	oRadarDiv.onmouseenter=handleMouseIn;
	mouseOutTimer=setTimeout(endMouseAction,2000);
}

function endMouseAction() {
	oRadarDiv.onmouseenter=null;
	initMouseEvent='';
	endDragging();
}

function handleMouseIn() {
	if (mouseOutTimer) clearTimeout(mouseOutTimer);
	oRadarDiv.onmouseenter=null;
}

/*	Reset dragging state to zero with no action, and remove selection box.
	We can show links when dragging is done */
function endDragging() {
	// Turn off dragging flag
	dragging=0;
	
	// Disable drag-related events
	oRadarDiv.onmouseleave=null;
	oRadarDiv.onmouseup=null;
	oRadarDiv.onmousemove=null;
	
	// Store new position if we were shifting image
	if (initMouseEvent=='shift') storeBox();
	
	// Hide selection box
	oSelectBox.style.visibility='hidden';
	
	// Show bottom links
	oLinkContainer.style.visibility='visible';
	oLinkShadow.style.visibility='visible';
}

/* When zooming out, load previous zoom state */
function radarUnZoom() {
	// We can only unzoom if there is a zoom state on the stack
	if (oZoomArray.length>0) {
		// Pop the last state off the zoom and get the total zoom from last element
		oCurrentBox=oZoomArray.pop();
		totalZoom=oCurrentBox[4];
		
		// Store current state and zoom to position
		storeBox();
	}
}

function radarZoom() {
	// We will allow a total zoom no greater than 40 or no selection box width
	if  ((totalZoom>=maxZoom) || ! oSelectBox.offsetWidth)  return;
	
	// Determine selection box based on selection box position
	selectLeft=oSelectBox.offsetLeft;
	selectTop=oSelectBox.offsetTop;
	selectWidth=oSelectBox.offsetWidth/cWinScale;
	selectHeight=oSelectBox.offsetHeight/cWinScale;
	selectMiddleX=selectLeft+selectWidth/2;
	selectMiddleY=selectTop+selectHeight/2;
	
	// Determine X- and Y-oriented zooms
	zoomLevelX=oRadarBackground.offsetWidth/selectWidth;
	zoomLevelY=oRadarBackground.offsetHeight/selectHeight;
					
	// Compare the radar image ratio to the radio of Y- to X- oriented zoom
	zoomRatio=zoomLevelY/zoomLevelX;
	zoomLevel=((zoomRatio<imgRadarRatio)?zoomLevelY:zoomLevelX)/cWinScale;
	
	// Calculate new zoom relative to current zoom, but cap maximum zoom
	maxZoomLevel=maxZoom/totalZoom;
	if (zoomLevel>maxZoomLevel) zoomLevel=maxZoomLevel;
	prevTotalZoom=totalZoom;
	totalZoom=totalZoom*zoomLevel;
	
	// Push old zoom state onto stack
	with (oRadarDiv) {
		zoomBox=new Array(
			parseInt(offsetLeft*cWinScale),
			parseInt(offsetTop*cWinScale),
			parseInt(offsetWidth*cWinScale),
			parseInt(offsetHeight*cWinScale),
			prevTotalZoom
		);
		oZoomArray.push(zoomBox);
	}
	
	// Determine new radarDiv positions based on new zoom level
	newLeft=-parseInt(zoomLevel*selectLeft*cWinScale);
	newTop=-parseInt(zoomLevel*selectTop*cWinScale);
	newWidth=parseInt(zoomLevel*oRadarDiv.offsetWidth*cWinScale);
	newHeight=parseInt(zoomLevel*oRadarDiv.offsetHeight*cWinScale);
		
	// Store new dimenstion state and reposition
	storeBox(newLeft,newTop,newWidth,newHeight,totalZoom);
}

/*	Write current box for next restart and zoom to position */
function storeBox(newLeft,newTop,newWidth,newHeight,newZoom) {
	// If a new current box was passed, update oCurrentBox
	if (newZoom>0)
		oCurrentBox=Array(newLeft,newTop,newWidth,newHeight,newZoom);

	// Make sure that we are zoomed to the right position
	zoomTo();
		
	// Store current gadget state
	System.Gadget.Settings.writeString("cBox0",oCurrentBox[0]);
	System.Gadget.Settings.writeString("cBox1",oCurrentBox[1]);
	System.Gadget.Settings.writeString("cBox2",oCurrentBox[2]);
	System.Gadget.Settings.writeString("cBox3",oCurrentBox[3]);
	System.Gadget.Settings.writeString("totalZoom",oCurrentBox[4]);
}

/* Move and scale radarDiv box according to current settings */
function zoomTo() {
	// Reposition and scale radar div
	with (oRadarDiv) {
		style.left=parseInt(oCurrentBox[0]/cWinScale);
		style.top=parseInt(oCurrentBox[1]/cWinScale);
		style.width=parseInt(oCurrentBox[2]/cWinScale);
		style.height=parseInt(oCurrentBox[3]/cWinScale);
	}
}

function selectMouseOver() {
	oRadarDiv.mouseout=null;
}

function selectMouseOut() {
	oRadarDiv.mouseout=handleMouseOut;
}
					
function overlayToggle(obj) {
	// Use object tag to determine appropriate state
	obj.tag=! obj.tag;
	if (obj.tag) obj.className='overlayOn';
	else obj.className='overlayOff';
	if (obj.id=='checkTopo') {
		oTopoImg.style.visibility=obj.tag?'visible':'hidden';
	} else if (obj.id=='checkWarnings') {
		oWarningImg.style.visibility=obj.tag?'visible':'hidden';
	} else {
		if (obj.tag) {
			resetOverlaySettings(false);
		} else {
			oCountyImg.style.visibility='hidden';
			oRiverImg.style.visibility='hidden';
			oHighwayImg.style.visibility='hidden';
			oCityImg.style.visibility='hidden';
		}
	}
}

/* If over the link area and not dragging, show the links */
function showNWSLinks() {
	if (dragging || (eulaAccept!='yes')) return;
	oLinkShadow.style.top=cInfoUp;
	oNWSLinks.style.top=0;
	oRadarLink.style.visibility='visible';
	oForecastLink.style.visibility='visible';
}

/* Hide the links area when appropriate */
function hideNWSLinks() {
	oRadarLink.style.visibility='hidden';
	oForecastLink.style.visibility='hidden';
	oNWSLinks.style.top=cInfoDown-cInfoUp;
	oLinkShadow.style.top=cInfoDown;
}

function loopStep() {
	var nextUpdate;
	
	// Make sure timer is clear
	if (loopTimer)
		clearTimeout(loopTimer);
		
	// If radar is updating or being changed, skip loop update, but check soon
	if (radarUpdating || dragging) {
		loopTimer=setTimeout(loopStep,100);
		
	// If no images, wait a few seconds
	} else if (oLoopRadar.length==0) {
		loopTimer=setTimeout(loopStep,3000)

	// Otherwise go ahead and update index
	} else {
		
		// If loop on, increment current index
		if (isLooping) {
			loopIndex=(loopIndex+1) % maxLoop;
			if (loopIndex>=oLoopRadar.length)	
				loopIndex=0;
			if (oLoopRadar.length>1) 
				nextUpdate=(loopIndex==(oLoopRadar.length-1))?parseInt(loopSpeed*2):loopSpeed;
			else
				nextUpdate=3000;
		
		// If not looping need to go to end of loop after no recent mouse clicks
		} else {
			epoch=new Date();
			if ((epoch.getTime()-lastClickTime>300) || (loopIndex>=oLoopRadar.length))
				loopIndex=oLoopRadar.length-1;
			else if (loopIndex<0) 
				loopIndex=0;
		}
	
		// Update image and label
		oRadarImg.src=oLoopRadar[loopIndex][4];
		oWarningImg.src=oLoopRadar[loopIndex][5];
		oInfo.innerHTML=oLoopRadar[loopIndex][6] ;
		if (isLooping) 
			loopTimer=setTimeout(loopStep,nextUpdate);
	}
}

/* When the play/pause button is clicked, toggle loop state */
function playToggle() {
	if (isLooping) {
		loopOff();
		gotoFrame(loopIndex);
	} else
		loopOn();
}

/* Change play button to pause and take first step when looping is switched on */
function loopOn() {
	isLooping=true;
	oPlayToggle.src='images/pause_button.gif';
	loopStep();
}

/* Change pause button to play and clear loop timer when looping is switched off */
function loopOff() {
	isLooping=false;
	if (loopTimer)
		clearTimeout(loopTimer);
	oPlayToggle.src='images/play_button.gif';
}

/* When step forward button is clicked, go forward one image in loop */
function stepForward() {
	newFrame=(loopIndex+1) % maxLoop;
	gotoFrame(newFrame);
}

/* When step backward button is clicked, go backward one image in loop */
function stepBack() {
	newFrame=loopIndex-1;
	if (newFrame<0) newFrame=oLoopRadar.length-1;
	gotoFrame(newFrame);
}

/* When head button is clicked, go to the first image in loop */
function head() {
	newFrame=0;
	gotoFrame(newFrame);
}

/* When tail button is clicked, go to last image in loop */
function tail() {
	newFrame=oLoopRadar.length-1;
	gotoFrame(newFrame);
}

/* Toggle visibility of help flyout when help button clicked */
function toggleHelp() {
	helpShowing=! helpShowing;
	System.Gadget.Flyout.show=helpShowing;
}

/*	When the flyout is made visible, insert information about the
	current script version */
function flyoutShowing() {
	oFlyoutDoc=System.Gadget.Flyout.document;
	with (oFlyoutDoc) {
		getElementById('currentVersion').innerHTML=thisVersion;
		if (newAvailable) {
			getElementById('newVersionInfo').style.visibility='visible';
			getElementById('newVersion').innerHTML=newAvailable;
		}
	}
}

/* When the flyout autmatically closes, toggle help showing flag */
function flyoutHiding() {
	helpShowing=false;
}

/* When we goto a specific frame, we also stop looping */
function gotoFrame(frameNum) {
	epoch=new Date();
	lastClickTime=epoch.getTime();
	if (isLooping) loopOff();
	if ((frameNum<0) || (frameNum>=oLoopRadar.length)) return;
	loopIndex=frameNum;
	oRadarImg.src=oLoopRadar[loopIndex][4];
	oWarningImg.src=oLoopRadar[loopIndex][5];
	oInfo.innerHTML=oLoopRadar[loopIndex][6];
}
